/**
 * LICENCIA LGPL:
 * 
 * Esta librera es Software Libre; Usted puede redistribuirla y/o modificarla
 * bajo los trminos de la GNU Lesser General Public License (LGPL) tal y como 
 * ha sido publicada por la Free Software Foundation; o bien la versin 2.1 de 
 * la Licencia, o (a su eleccin) cualquier versin posterior.
 * 
 * Esta librera se distribuye con la esperanza de que sea til, pero SIN 
 * NINGUNA GARANTA; tampoco las implcitas garantas de MERCANTILIDAD o 
 * ADECUACIN A UN PROPSITO PARTICULAR. Consulte la GNU Lesser General Public 
 * License (LGPL) para ms detalles
 * 
 * Usted debe recibir una copia de la GNU Lesser General Public License (LGPL) 
 * junto con esta librera; si no es as, escriba a la Free Software Foundation 
 * Inc. 51 Franklin Street, 5 Piso, Boston, MA 02110-1301, USA o consulte
 * <http://www.gnu.org/licenses/>.
 *
 * Copyright 2011 Agencia de Tecnologa y Certificacin Electrnica
 */
package es.accv.arangi.certificate;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.security.cert.X509Certificate;

import es.accv.arangi.base.certificate.validation.CAList;
import es.accv.arangi.base.certificate.validation.ValidateCertificate;
import es.accv.arangi.base.exception.certificate.CertificateCANotFoundException;
import es.accv.arangi.base.exception.certificate.NormalizeCertificateException;

/**
 * Certificado desconocido para Arang
 * 
 * @author <a href="mailto:jgutierrez@accv.es">Jos M Gutirrez</a>
 */
public class CertificadoDesconocido extends ValidateCertificate {

	/**
	 * Constructor con un certificado X509Certificate
	 * 
	 * @param certificate Certificado en formato X.509
	 * @param caList Lista de certificados de CA
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoDesconocido(X509Certificate certificate, CAList caList) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(certificate, caList);
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param fileCertificate Fichero que contiene un certificado en formato X.509
	 * @param caList Lista de certificados de CA
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 * @throws FileNotFoundException El fichero no existe
	 */
	public CertificadoDesconocido(File fileCertificate, CAList caList) throws CertificateCANotFoundException, NormalizeCertificateException, FileNotFoundException {
		super(fileCertificate, caList);
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param isCertificate Stream de lectura a un certificado en formato X.509
	 * @param caList Lista de certificados de CA
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoDesconocido(InputStream isCertificate, CAList caList) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(isCertificate, caList);
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param contenidoCertificado Contenido de un certificado en formato X.509
	 * @param caList Lista de certificados de CA
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoDesconocido(byte[] contenidoCertificado, CAList caList) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(contenidoCertificado, caList);
	}
	

}
